import imageio
import numpy as np
import matplotlib.pyplot as plt
import os
from os.path import dirname, join
from _util import make_gif

class Gif:
    """
    All inclusive Gif object.

    Parameters:
        path: string
            String which a temporary directory will save frames as .png images before combining to a standalone .gif.
    """
    def __init__(self, path:str):
        self.path = path
        self.tempdir = join(path, "temp_img")
        self.i = 0
        self._make_temp_dir()

    def _make_temp_dir(self):
        """ Creates a temporary image directory """
        try:
            os.makedirs(self.tempdir)
        except OSError as e:
            print("Error: %s : %s" % (self.tempdir, e.strerror))

    def _remove_temp_dir(self):
        """ Destroys the temporary image directory """
        try:
            # remove contents of temp directory then the directory itself
            for elem in os.listdir(self.tempdir):
                os.remove(join(self.tempdir, elem))
            os.rmdir(self.tempdir)
        except OSError as e:
            print("Error: %s : %s" % (self.tempdir, e.strerror))

    def frame(self) -> None:
        """
        Add a frame to the gif! Use this function inside a plot-updating loop.

        Utilizes matplotlib's pyplot.savefig() to temporarily save images from a iterating frame.
        """
        plt.savefig(join(self.tempdir, "img (" + str(self.i) + ").png"))
        self.i += 1

    def save(self, gifname:str) -> None:
        """ 
        Save and combine .png's into a .gif.
        Utilizes this modules make_gif() function.

        Parameters:
            gifname: string
                The final name of the constructed gif-file. Use .gif at the end.
        """
        make_gif(self.tempdir, join(gifname))
        self._remove_temp_dir()


import imageio
import os

def make_gif(folder_name: str, save_as: str) -> None:
    """ 
    Makes a gif from a collection of png images in an array where image names are of the form: img (i).png for i = 0, i -> N.

    Parameters:
        folder_name: string
            Name of the folder containing images which are to be converted into a gif.
        save_as: string
            Name of final gif file, with absolute path. Must be of the form: ../my_video.gif
    """

    filenames = ["img (" + str(i) + ").png" for i, _ in enumerate(os.listdir(folder_name))]
    images = []
    for filename in filenames:
        images.append(imageio.imread(os.path.join(folder_name, filename)))

    imageio.mimsave(save_as, images)



import imageio as im
import numpy as np
from os.path import join
from os import getcwd, listdir

class Converter:
    def __init__(self, folder_path):
        self.folder_path = folder_path
        self.filenames = self._fix_order(listdir(folder_path))
    
    def _fix_order(self, filenames):
        res = []

        for i, filename in enumerate(filenames):
            str1, str2 = filename.split("(")
            image_index, str22 = str2.split(")")

            res.append(int(image_index))
        
        numpy_filenames = np.asarray(filenames)
        sorted_result = numpy_filenames[np.argsort(res)]
        return sorted_result

    def covert(self, save_as):
        images = []
        for filename in self.filenames:
            images.append(im.imread(filename))
        im.mimsave(save_as, images)

    def _is_string(self, string):
        pass

def main():
    path = join(getcwd(), "test_img")
    ins = Converter(path)

if __name__ == "__main__":
    main()


from setuptools import setup, find_packages

VERSION = "0.0.3"
DESCRIPTION = "Basic .gif converter"

with open("README.md", "r", encoding="utf-8") as fh:
    LONG_DESCRIPTION = fh.read()

setup(
    name="giffer",
    version=VERSION,
    author="Salomaestro",
    author_email="<chris10an.salomonsen@gmail.com>",
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    url="https://github.com/salomaestro/giffer",
    packages=find_packages(),
    install_requires=["imageio", "numpy", "Pillow"],
    keywords=["python", "basic", "gif"],
    classifiers=[
        "Operating System :: Microsoft :: Windows",
        "Operating System :: MacOS :: MacOS X"
    ]
)


import numpy as np
import matplotlib.pyplot as plt
from giffer import Gif
import os

# set a path
path = os.path.dirname(__file__)

# initialize gif object
gif = Gif(path)

# construct plots
fig, axs = plt.subplots(2, 1)

x = np.arange(100)
for i in range(10):

    # clear axes
    axs[0].cla()
    axs[1].cla()

    y1 = np.sin(x * i)
    y2 = np.cos(x * i)
    
    axs[0].plot(x, y1)
    axs[1].plot(x, y2)
    
    # make animation
    plt.pause(0.1)

    # capture frame
    gif.frame()

# save and combine to a .gif
gif.save("my_gif.gif")